=== modified file 'NEWS'
--- NEWS	2006-01-31 15:33:47 +0000
+++ NEWS	2006-02-06 14:02:09 +0000
@@ -30,6 +30,11 @@
     * SFTP tests now shortcut the SSH negotiation, reducing test overhead
       for testing SFTP protocol support. (Robey Pointer)
 
+  IMPROVEMENTS:
+
+    * pull now accepts a --revision argument (Erik Bågfors)
+
+
 bzr 0.7rc1 2006-01-09
 
   CHANGES:

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2006-02-01 12:46:12 +0000
+++ bzrlib/branch.py	2006-02-13 21:40:27 +0000
@@ -334,7 +334,7 @@
         """Return a `Tree` for the working copy if this is a local branch."""
         raise NotImplementedError('working_tree is abstract')
 
-    def pull(self, source, overwrite=False):
+    def pull(self, source, overwrite=False, stop_revision=None):
         raise NotImplementedError('pull is abstract')
 
     def basis_tree(self):
@@ -430,7 +430,7 @@
             os.mkdir(to_location)
         if to_branch_type is None:
             to_branch_type = BzrBranch
-        print "FIXME use a branch format here"
+        # FIXME use a branch format here
         br_to = to_branch_type.initialize(to_location)
         mutter("copy branch from %s to %s", self, br_to)
         if basis_branch is not None:
@@ -933,6 +933,7 @@
     def update_revisions(self, other, stop_revision=None):
         """See Branch.update_revisions."""
         from bzrlib.fetch import greedy_fetch
+
         if stop_revision is None:
             stop_revision = other.last_revision()
         ### Should this be checking is_ancestor instead of revision_history?
@@ -987,13 +988,13 @@
         return WorkingTree(self.base, branch=self)
 
     @needs_write_lock
-    def pull(self, source, overwrite=False):
+    def pull(self, source, overwrite=False, stop_revision=None):
         """See Branch.pull."""
         source.lock_read()
         try:
             old_count = len(self.revision_history())
             try:
-                self.update_revisions(source)
+                self.update_revisions(source,stop_revision)
             except DivergedBranches:
                 if not overwrite:
                     raise
@@ -1076,7 +1077,8 @@
         return branch_to
 
     def clone(self, to_location, revision=None, basis_branch=None, to_branch_type=None):
-        print "FIXME: clone via create and fetch is probably faster when versioned file comes in."
+        # FIXME: clone via create and fetch is probably faster when versioned
+        # file comes in.
         if to_branch_type is None:
             to_branch_type = BzrBranch
 

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2006-02-01 14:13:39 +0000
+++ bzrlib/builtins.py	2006-02-13 21:40:27 +0000
@@ -392,10 +392,10 @@
     If you want to forget your local changes and just update your branch to
     match the remote one, use --overwrite.
     """
-    takes_options = ['remember', 'overwrite', 'verbose']
+    takes_options = ['remember', 'overwrite', 'revision', 'verbose']
     takes_args = ['location?']
 
-    def run(self, location=None, remember=False, overwrite=False, verbose=False):
+    def run(self, location=None, remember=False, overwrite=False, revision=None, verbose=False):
         from shutil import rmtree
         import errno
         # FIXME: too much stuff is in the command class        
@@ -411,8 +411,15 @@
         br_from = Branch.open(location)
         br_to = tree_to.branch
 
+        if revision is None:
+            rev_id = None
+        elif len(revision) == 1:
+            rev_id = revision[0].in_history(br_from).rev_id
+        else:
+            raise BzrCommandError('bzr pull --revision takes one value.')
+
         old_rh = br_to.revision_history()
-        count = tree_to.pull(br_from, overwrite)
+        count = tree_to.pull(br_from, overwrite, rev_id)
 
         if br_to.get_parent() is None or remember:
             br_to.set_parent(location)
@@ -942,7 +949,11 @@
             rev1 = rev2 = revision[0].in_history(b).revno
         elif len(revision) == 2:
             rev1 = revision[0].in_history(b).revno
-            rev2 = revision[1].in_history(b).revno
+            if revision[1].spec is None:
+                # missing end-range means last known revision
+                rev2 = b.revno()
+            else:
+                rev2 = revision[1].in_history(b).revno
         else:
             raise BzrCommandError('bzr log --revision takes one or two values.')
 

=== modified file 'bzrlib/commands.py'
--- bzrlib/commands.py	2006-02-01 14:13:39 +0000
+++ bzrlib/commands.py	2006-02-04 23:41:45 +0000
@@ -447,11 +447,17 @@
         os.remove(pfname)
 
 
-def apply_lsprofiled(the_callable, *args, **kwargs):
+def apply_lsprofiled(filename, the_callable, *args, **kwargs):
     from bzrlib.lsprof import profile
-    ret,stats = profile(the_callable,*args,**kwargs)
+    import cPickle
+    ret, stats = profile(the_callable, *args, **kwargs)
     stats.sort()
-    stats.pprint()
+    if filename is None:
+        stats.pprint()
+    else:
+        stats.freeze()
+        cPickle.dump(stats, open(filename, 'w'), 2)
+        print 'Profile data written to %r.' % filename
     return ret
 
 def run_bzr(argv):
@@ -484,16 +490,23 @@
     argv = [a.decode(bzrlib.user_encoding) for a in argv]
 
     opt_lsprof = opt_profile = opt_no_plugins = opt_builtin = False
+    opt_lsprof_file = None
 
     # --no-plugins is handled specially at a very early stage. We need
     # to load plugins before doing other command parsing so that they
     # can override commands, but this needs to happen first.
 
-    for a in argv:
+    argv_copy = []
+    i = 0
+    while i < len(argv):
+        a = argv[i]
         if a == '--profile':
             opt_profile = True
         elif a == '--lsprof':
             opt_lsprof = True
+        elif a == '--lsprof-file':
+            opt_lsprof_file = argv[i + 1]
+            i += 1
         elif a == '--no-plugins':
             opt_no_plugins = True
         elif a == '--builtin':
@@ -501,9 +514,10 @@
         elif a in ('--quiet', '-q'):
             be_quiet()
         else:
-            continue
-        argv.remove(a)
+            argv_copy.append(a)
+        i += 1
 
+    argv = argv_copy
     if (not argv) or (argv[0] == '--help'):
         from bzrlib.help import help
         if len(argv) > 1:
@@ -530,7 +544,7 @@
 
     try:
         if opt_lsprof:
-            ret = apply_lsprofiled(cmd_obj.run_argv, argv)
+            ret = apply_lsprofiled(opt_lsprof_file, cmd_obj.run_argv, argv)
         elif opt_profile:
             ret = apply_profiled(cmd_obj.run_argv, argv)
         else:

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2006-02-01 14:13:39 +0000
+++ bzrlib/errors.py	2006-02-13 21:40:27 +0000
@@ -301,18 +301,21 @@
             " specified."
         BzrCommandError.__init__(self, msg)
 
+
 class NoCommonAncestor(BzrError):
     def __init__(self, revision_a, revision_b):
         msg = "Revisions have no common ancestor: %s %s." \
             % (revision_a, revision_b) 
         BzrError.__init__(self, msg)
 
+
 class NoCommonRoot(BzrError):
     def __init__(self, revision_a, revision_b):
         msg = "Revisions are not derived from the same root: %s %s." \
             % (revision_a, revision_b) 
         BzrError.__init__(self, msg)
 
+
 class NotAncestor(BzrError):
     def __init__(self, rev_id, not_ancestor_id):
         msg = "Revision %s is not an ancestor of %s" % (not_ancestor_id, 
@@ -336,15 +339,18 @@
         BzrError.__init__(self, msg)
         self.bases = bases
 
+
 class NoCommits(BzrError):
     def __init__(self, branch):
         msg = "Branch %s has no commits." % branch
         BzrError.__init__(self, msg)
 
+
 class UnlistableStore(BzrError):
     def __init__(self, store):
         BzrError.__init__(self, "Store %s is not listable" % store)
 
+
 class UnlistableBranch(BzrError):
     def __init__(self, br):
         BzrError.__init__(self, "Stores for branch %s are not listable" % br)
@@ -416,6 +422,7 @@
         self.msg = msg
         self.orig_error = orig_error
 
+
 # A set of semi-meaningful errors which can be thrown
 class TransportNotPossible(TransportError):
     """This is for transports where a specific function is explicitly not
@@ -437,10 +444,12 @@
     """The connection has been closed."""
     pass
 
+
 class ConflictsInTree(BzrError):
     def __init__(self):
         BzrError.__init__(self, "Working tree has conflicts.")
 
+
 class ParseConfigError(BzrError):
     def __init__(self, errors, filename):
         if filename is None:
@@ -449,21 +458,25 @@
             (filename, ('\n'.join(e.message for e in errors)))
         BzrError.__init__(self, message)
 
+
 class SigningFailed(BzrError):
     def __init__(self, command_line):
         BzrError.__init__(self, "Failed to gpg sign data with command '%s'"
                                % command_line)
 
+
 class WorkingTreeNotRevision(BzrError):
     def __init__(self, tree):
         BzrError.__init__(self, "The working tree for %s has changed since"
                           " last commit, but weave merge requires that it be"
                           " unchanged." % tree.basedir)
 
+
 class CantReprocessAndShowBase(BzrNewError):
     """Can't reprocess and show base.
 Reprocessing obscures relationship of conflicting lines to base."""
 
+
 class GraphCycleError(BzrNewError):
     """Cycle in graph %(graph)r"""
     def __init__(self, graph):

=== modified file 'bzrlib/fetch.py'
--- bzrlib/fetch.py	2006-01-30 01:13:27 +0000
+++ bzrlib/fetch.py	2006-02-12 19:35:02 +0000
@@ -163,7 +163,7 @@
         count = 0
         num_file_ids = len(file_ids)
         for file_id in file_ids:
-            self.pb.update("merge weave merge", count, num_file_ids)
+            self.pb.update("merge weaves", count, num_file_ids)
             count +=1
             to_weave = self.to_weaves.get_weave_or_empty(file_id,
                 self.to_branch.get_transaction())

=== modified file 'bzrlib/lockable_files.py'
--- bzrlib/lockable_files.py	2006-01-26 20:52:58 +0000
+++ bzrlib/lockable_files.py	2006-02-13 19:34:47 +0000
@@ -162,7 +162,7 @@
         self.put(path, StringIO(a_string.encode('utf-8')))
 
     def lock_write(self):
-        mutter("lock write: %s (%s)", self, self._lock_count)
+        # mutter("lock write: %s (%s)", self, self._lock_count)
         # TODO: Upgrade locking to support using a Transport,
         # and potentially a remote locking protocol
         if self._lock_mode:
@@ -178,7 +178,7 @@
             self._set_transaction(transactions.PassThroughTransaction())
 
     def lock_read(self):
-        mutter("lock read: %s (%s)", self, self._lock_count)
+        # mutter("lock read: %s (%s)", self, self._lock_count)
         if self._lock_mode:
             assert self._lock_mode in ('r', 'w'), \
                    "invalid lock mode %r" % self._lock_mode
@@ -193,7 +193,7 @@
             self.get_transaction().set_cache_size(5000)
                         
     def unlock(self):
-        mutter("unlock: %s (%s)", self, self._lock_count)
+        # mutter("unlock: %s (%s)", self, self._lock_count)
         if not self._lock_mode:
             raise LockError('branch %r is not locked' % (self))
 

=== modified file 'bzrlib/status.py'
--- bzrlib/status.py	2005-12-25 00:38:48 +0000
+++ bzrlib/status.py	2006-02-04 23:41:45 +0000
@@ -74,7 +74,7 @@
                 old = branch.repository.revision_tree(rev_id)
             except NoSuchRevision, e:
                 raise BzrCommandError(str(e))
-            if len(revision) > 1:
+            if (len(revision) > 1) and (revision[1].spec is not None):
                 try:
                     rev_id = revision[1].in_history(branch).rev_id
                     new = branch.repository.revision_tree(rev_id)

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2006-02-01 14:13:39 +0000
+++ bzrlib/tests/__init__.py	2006-02-13 21:40:27 +0000
@@ -83,6 +83,7 @@
     import bzrlib.tests.branch_implementations
     return [
             bzrlib.doc,
+            bzrlib.tests.blackbox,
             bzrlib.tests.branch_implementations,
             ]
 

=== modified file 'bzrlib/tests/blackbox/__init__.py'
--- bzrlib/tests/blackbox/__init__.py	2006-02-01 14:13:39 +0000
+++ bzrlib/tests/blackbox/__init__.py	2006-02-13 21:40:27 +0000
@@ -23,8 +23,8 @@
 rather starts again from the run_bzr function.
 """
 
-from bzrlib.tests import TestLoader
-from bzrlib.tests import TestCaseInTempDir, BzrTestBase
+from bzrlib.tests import (TestLoader, TestSuite, _load_module_by_name,
+                          TestCaseInTempDir, BzrTestBase)
 
 def test_suite():
     testmod_names = [
@@ -34,6 +34,7 @@
                      'bzrlib.tests.blackbox.test_diff',
                      'bzrlib.tests.blackbox.test_export',
                      'bzrlib.tests.blackbox.test_find_merge_base',
+                     'bzrlib.tests.blackbox.test_log',
                      'bzrlib.tests.blackbox.test_missing',
                      'bzrlib.tests.blackbox.test_outside_wt',
                      'bzrlib.tests.blackbox.test_pull',
@@ -46,7 +47,13 @@
                      'bzrlib.tests.blackbox.test_upgrade',
                      'bzrlib.tests.blackbox.test_versioning',
                      ]
-    return TestLoader().loadTestsFromNames(testmod_names)
+
+    suite = TestSuite()
+    loader = TestLoader()
+    for mod_name in testmod_names:
+        mod = _load_module_by_name(mod_name)
+        suite.addTest(loader.loadTestsFromModule(mod))
+    return suite
 
 
 class ExternalBase(TestCaseInTempDir):

=== added file 'bzrlib/tests/blackbox/test_log.py'
--- bzrlib/tests/blackbox/test_log.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_log.py	2006-01-12 09:03:06 +0000
@@ -0,0 +1,40 @@
+# Copyright (C) 2005 by Canonical Ltd
+# -*- coding: utf-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Black-box tests for bzr log.
+"""
+
+import os
+
+import bzrlib
+from bzrlib.tests.blackbox import ExternalBase
+
+
+class TestLog(ExternalBase):
+
+    def test_log(self):
+        self.runbzr("init")
+        self.build_tree(['hello.txt'])
+        self.runbzr("add hello.txt")
+        self.runbzr("commit -m message hello.txt")
+        result = self.runbzr("log")[0]
+        self.assertTrue('revno: 1\n' in result)
+        self.assertTrue('message:\n  message\n' in result)
+        
+        result2 = self.runbzr("log -r 1..")[0]
+        self.assertEquals(result2, result)

=== modified file 'bzrlib/tests/blackbox/test_pull.py'
--- bzrlib/tests/blackbox/test_pull.py	2005-12-01 21:49:24 +0000
+++ bzrlib/tests/blackbox/test_pull.py	2006-01-17 11:44:03 +0000
@@ -101,6 +101,33 @@
         self.runbzr('pull ../b')
         self.runbzr('pull ../b')
 
+    def test_pull_revision(self):
+        """Pull some changes from one branch to another."""
+        os.mkdir('a')
+        os.chdir('a')
+
+        self.example_branch()
+        file('hello2', 'wt').write('foo')
+        self.runbzr('add hello2')
+        self.runbzr('commit -m setup hello2')
+        file('goodbye2', 'wt').write('baz')
+        self.runbzr('add goodbye2')
+        self.runbzr('commit -m setup goodbye2')
+
+        os.chdir('..')
+        self.runbzr('branch -r 1 a b')
+        os.chdir('b')
+        self.runbzr('pull -r 2')
+        a = Branch.open('../a')
+        b = Branch.open('.')
+        self.assertEquals(a.revno(),4)
+        self.assertEquals(b.revno(),2)
+        self.runbzr('pull -r 3')
+        self.assertEquals(b.revno(),3)
+        self.runbzr('pull -r 4')
+        self.assertEquals(a.revision_history(), b.revision_history())
+
+
     def test_overwrite_uptodate(self):
         # Make sure pull --overwrite overwrites
         # even if the target branch has merged

=== modified file 'bzrlib/tests/blackbox/test_status.py'
--- bzrlib/tests/blackbox/test_status.py	2006-01-26 20:52:58 +0000
+++ bzrlib/tests/blackbox/test_status.py	2006-02-13 21:40:27 +0000
@@ -29,8 +29,6 @@
 from tempfile import TemporaryFile
 import codecs
 
-from bzrlib.clone import copy_branch
-from bzrlib.branch import Branch
 from bzrlib.builtins import merge
 from bzrlib.revisionspec import RevisionSpec
 from bzrlib.status import show_status
@@ -173,3 +171,27 @@
                           ['unknown:\n',
                            '  dir2\n'
                            ])
+
+
+class TestStatus(TestCaseInTempDir):
+
+    def test_status(self):
+        self.run_bzr("init")
+        self.build_tree(['hello.txt'])
+        result = self.run_bzr("status")[0]
+        self.assert_("unknown:\n  hello.txt\n" in result, result)
+        self.run_bzr("add", "hello.txt")
+        result = self.run_bzr("status")[0]
+        self.assert_("added:\n  hello.txt\n" in result, result)
+        self.run_bzr("commit", "-m", "added")
+        result = self.run_bzr("status", "-r", "0..1")[0]
+        self.assert_("added:\n  hello.txt\n" in result, result)
+        self.build_tree(['world.txt'])
+        result = self.run_bzr("status", "-r", "0")[0]
+        self.assert_("added:\n  hello.txt\n" \
+                     "unknown:\n  world.txt\n" in result, result)
+
+        result2 = self.run_bzr("status", "-r", "0..")[0]
+        self.assertEquals(result2, result)
+
+

=== modified file 'bzrlib/tests/blackbox/test_too_much.py'
--- bzrlib/tests/blackbox/test_too_much.py	2006-01-26 20:52:58 +0000
+++ bzrlib/tests/blackbox/test_too_much.py	2006-02-04 23:41:45 +0000
@@ -241,22 +241,6 @@
         self.runbzr('revert')
         os.chdir('..')
 
-    def test_status(self):
-        self.runbzr("init")
-        self.build_tree(['hello.txt'])
-        result = self.runbzr("status")
-        self.assert_("unknown:\n  hello.txt\n" in result, result)
-        self.runbzr("add hello.txt")
-        result = self.runbzr("status")
-        self.assert_("added:\n  hello.txt\n" in result, result)
-        self.runbzr("commit -m added")
-        result = self.runbzr("status -r 0..1")
-        self.assert_("added:\n  hello.txt\n" in result, result)
-        self.build_tree(['world.txt'])
-        result = self.runbzr("status -r 0")
-        self.assert_("added:\n  hello.txt\n" \
-                     "unknown:\n  world.txt\n" in result, result)
-
     def test_mv_modes(self):
         """Test two modes of operation for mv"""
         self.runbzr('init')

=== modified file 'bzrlib/tests/test_fileid_involved.py'
--- bzrlib/tests/test_fileid_involved.py	2006-02-01 12:24:35 +0000
+++ bzrlib/tests/test_fileid_involved.py	2006-02-13 19:43:22 +0000
@@ -26,8 +26,7 @@
 from bzrlib.tests import TestCaseWithTransport
 from bzrlib.workingtree import WorkingTree
 
-
-class TestFileIdInvolved(TestCaseWithTransport):
+class FileIdInvolvedBase(TestCaseWithTransport):
 
     def touch(self,filename):
         f = file(filename,"a")
@@ -46,6 +45,21 @@
                     this_tree=wt_to)
         wt_to.add_pending_merge(branch_from.last_revision())
 
+    def compare_tree_fileids(self, branch, old_rev, new_rev):
+        old_tree = self.branch.repository.revision_tree(old_rev)
+        new_tree = self.branch.repository.revision_tree(new_rev)
+        delta = compare_trees(old_tree, new_tree)
+
+        l2 = [id for path, id, kind in delta.added] + \
+             [id for oldpath, newpath, id, kind, text_modified, \
+                meta_modified in delta.renamed] + \
+             [id for path, id, kind, text_modified, meta_modified in \
+                delta.modified]
+        return set(l2)
+
+    
+class TestFileIdInvolved(FileIdInvolvedBase):
+
     def setUp(self):
         super(TestFileIdInvolved, self).setUp()
         # create three branches, and merge it
@@ -166,21 +180,48 @@
         if len(history) < 2: return
 
         for start in range(0,len(history)-1):
+            start_id = history[start]
             for end in range(start+1,len(history)):
-
-                l1 = self.branch.fileid_involved_between_revs(
-                    history[start], history[end])
-
-                old_tree = self.branch.repository.revision_tree(history[start])
-                new_tree = self.branch.repository.revision_tree(history[end])
-                delta = compare_trees(old_tree, new_tree )
-
-                l2 = [id for path, id, kind in delta.added] + \
-                     [id for oldpath, newpath, id, kind, text_modified, \
-                        meta_modified in delta.renamed] + \
-                     [id for path, id, kind, text_modified, meta_modified in \
-                        delta.modified]
-
-                self.assertEquals(l1, set(l2))
-
-
+                end_id = history[end]
+                l1 = self.branch.fileid_involved_between_revs(start_id, end_id)
+
+                l2 = self.compare_tree_fileids(self.branch, start_id, end_id)
+                self.assertEquals(l1, l2)
+
+
+class TestFileIdInvolvedSuperset(FileIdInvolvedBase):
+
+    def setUp(self):
+        super(TestFileIdInvolvedSuperset, self).setUp()
+
+        main_wt = self.make_branch_and_tree('main')
+        main_branch = main_wt.branch
+        self.build_tree(["main/a","main/b","main/c"])
+
+        main_wt.add(['a', 'b', 'c'], ['a-file-id-2006-01-01-abcd',
+                                 'b-file-id-2006-01-01-defg',
+                                 'c-funky<file-id> quiji%bo'])
+        main_wt.commit("Commit one", rev_id="rev-A")
+
+        branch2_branch = main_branch.clone("branch2")
+        os.chmod("branch2/b",0770)
+        branch2_branch.working_tree().commit("branch2, Commit one", 
+                                             rev_id="rev-J")
+
+        self.merge(branch2_branch, main_wt)
+        os.chmod("main/b",0660)
+        main_wt.commit("merge branch1, rev-22",  rev_id="rev-G")
+
+        # end G
+        self.branch = main_branch
+
+    def test_fileid_involved_full_compare2(self):
+        history = self.branch.revision_history()
+        old_rev = history[0]
+        new_rev = history[1]
+
+        l1 = self.branch.fileid_involved_between_revs(old_rev, new_rev)
+
+        l2 = self.compare_tree_fileids(self.branch, old_rev, new_rev)
+        self.assertNotEqual(l2, l1)
+        self.AssertSubset(l2, l1)

=== modified file 'bzrlib/transport/http.py'
--- bzrlib/transport/http.py	2006-01-20 09:06:19 +0000
+++ bzrlib/transport/http.py	2006-02-04 23:41:45 +0000
@@ -22,12 +22,14 @@
 import urlparse
 from warnings import warn
 
+import bzrlib
 from bzrlib.transport import Transport, Server
 from bzrlib.errors import (TransportNotPossible, NoSuchFile, 
                            TransportError, ConnectionError)
 from bzrlib.errors import BzrError, BzrCheckError
 from bzrlib.branch import Branch
 from bzrlib.trace import mutter
+from bzrlib.ui import ui_factory
 
 
 def extract_auth(url, password_manager):
@@ -36,40 +38,42 @@
     password manager.  Return the url, minus those auth parameters (which
     confuse urllib2).
     """
-    assert url.startswith('http://') or url.startswith('https://')
-    scheme, host = url.split('//', 1)
-    if '/' in host:
-        host, path = host.split('/', 1)
-        path = '/' + path
-    else:
-        path = ''
-    port = ''
-    if '@' in host:
-        auth, host = host.split('@', 1)
+    scheme, netloc, path, query, fragment = urlparse.urlsplit(url)
+    assert (scheme == 'http') or (scheme == 'https')
+    
+    if '@' in netloc:
+        auth, netloc = netloc.split('@', 1)
         if ':' in auth:
             username, password = auth.split(':', 1)
         else:
             username, password = auth, None
-        if ':' in host:
-            host, port = host.split(':', 1)
-            port = ':' + port
-        # FIXME: if password isn't given, should we ask for it?
+        if ':' in netloc:
+            host = netloc.split(':', 1)[0]
+        else:
+            host = netloc
+        username = urllib.unquote(username)
         if password is not None:
-            username = urllib.unquote(username)
             password = urllib.unquote(password)
-            password_manager.add_password(None, host, username, password)
-    url = scheme + '//' + host + port + path
+        else:
+            password = ui_factory.get_password(prompt='HTTP %(user)@%(host) password',
+                                               user=username, host=host)
+        password_manager.add_password(None, host, username, password)
+    url = urlparse.urlunsplit((scheme, netloc, path, query, fragment))
     return url
+
     
 def get_url(url):
     import urllib2
-    mutter("get_url %s" % url)
+    mutter("get_url %s", url)
     manager = urllib2.HTTPPasswordMgrWithDefaultRealm()
     url = extract_auth(url, manager)
     auth_handler = urllib2.HTTPBasicAuthHandler(manager)
     opener = urllib2.build_opener(auth_handler)
-    url_f = opener.open(url)
-    return url_f
+
+    request = urllib2.Request(url)
+    request.add_header('User-Agent', 'bzr/%s' % bzrlib.__version__)
+    response = opener.open(request)
+    return response
 
 class HttpTransport(Transport):
     """This is the transport agent for http:// access.

=== modified file 'bzrlib/transport/sftp.py'
--- bzrlib/transport/sftp.py	2006-02-01 12:24:35 +0000
+++ bzrlib/transport/sftp.py	2006-02-04 23:41:45 +0000
@@ -19,14 +19,14 @@
 import errno
 import getpass
 import os
+import random
 import re
 import stat
+import subprocess
 import sys
+import time
 import urllib
 import urlparse
-import time
-import random
-import subprocess
 import weakref
 
 from bzrlib.config import config_dir, ensure_config_dir_exists
@@ -53,7 +53,13 @@
     from paramiko.sftp_file import SFTPFile
     from paramiko.sftp_client import SFTPClient
 
-if 'sftp' not in urlparse.uses_netloc: urlparse.uses_netloc.append('sftp')
+if 'sftp' not in urlparse.uses_netloc:
+    urlparse.uses_netloc.append('sftp')
+
+# don't use prefetch unless paramiko version >= 1.5.2 (there were bugs earlier)
+_default_do_prefetch = False
+if getattr(paramiko, '__version_info__', (0, 0, 0)) >= (1, 5, 2):
+    _default_do_prefetch = True
 
 
 _close_fds = True
@@ -62,6 +68,7 @@
     _close_fds = False
 
 _ssh_vendor = None
+
 def _get_ssh_vendor():
     """Find out what version of SSH is on the system."""
     global _ssh_vendor
@@ -70,6 +77,12 @@
 
     _ssh_vendor = 'none'
 
+    if 'BZR_SSH' in os.environ:
+        _ssh_vendor = os.environ['BZR_SSH']
+        if _ssh_vendor == 'paramiko':
+            _ssh_vendor = 'none'
+        return _ssh_vendor
+
     try:
         p = subprocess.Popen(['ssh', '-V'],
                              close_fds=_close_fds,
@@ -243,12 +256,11 @@
             # What specific errors should we catch here?
             pass
 
-
 class SFTPTransport (Transport):
     """
     Transport implementation for SFTP access.
     """
-    _do_prefetch = False # Right now Paramiko's prefetch support causes things to hang
+    _do_prefetch = _default_do_prefetch
 
     def __init__(self, base, clone_from=None):
         assert base.startswith('sftp://')
@@ -353,7 +365,7 @@
         try:
             path = self._remote_path(relpath)
             f = self._sftp.file(path, mode='rb')
-            if self._do_prefetch and hasattr(f, 'prefetch'):
+            if self._do_prefetch and (getattr(f, 'prefetch', None) is not None):
                 f.prefetch()
             return f
         except (IOError, paramiko.SSHException), e:

=== modified file 'bzrlib/upgrade.py'
--- bzrlib/upgrade.py	2006-01-26 22:15:04 +0000
+++ bzrlib/upgrade.py	2006-02-13 19:35:41 +0000
@@ -109,7 +109,7 @@
         # logic becomes format specific, then we can have the format know how to pass this
         # on. Also note that we probably have an 'upgrade meta' which upgrades the constituent
         # parts.
-        print "FIXME: control files reuse" 
+        # FIXME: control files reuse
         self.control_files.lock_write()
         try:
             self.convert()
@@ -219,7 +219,7 @@
 
     def _cleanup_spare_files_after_format4(self):
         transport = self.transport.clone('.bzr')
-        print "FIXME working tree upgrade foo."
+        # FIXME working tree upgrade foo.
         for n in 'merged-patches', 'pending-merged-patches':
             try:
                 ## assert os.path.getsize(p) == 0
@@ -242,7 +242,7 @@
         branch = self.branch
         inv = serializer_v4.read_inventory(branch.control_files.get('inventory'))
         new_inv_xml = serializer_v5.write_inventory_to_string(inv)
-        print "fixme inventory is a working tree change."
+        # FIXME inventory is a working tree change.
         branch.control_files.put('inventory', new_inv_xml)
 
     def _write_all_weaves(self):

=== modified file 'bzrlib/weave.py'
--- bzrlib/weave.py	2006-01-31 14:09:11 +0000
+++ bzrlib/weave.py	2006-02-12 19:35:02 +0000
@@ -819,43 +819,45 @@
         # two loops so that we do not change ourselves before verifying it
         # will be ok
         # work through in index order to make sure we get all dependencies
+        names_to_join = []
+        processed = 0
         for other_idx, name in enumerate(other._names):
             self._check_version_consistent(other, other_idx, name)
+            sha1 = other._sha1s[other_idx]
+
+            processed += 1
+
+            if name in self._name_map:
+                idx = self.lookup(name)
+                n1 = set(map(other.idx_to_name, other._parents[other_idx]))
+                n2 = set(map(self.idx_to_name, self._parents[idx]))
+                if sha1 ==  self._sha1s[idx] and n1 == n2:
+                        continue
+
+            names_to_join.append((other_idx, name))
 
         if pb and not msg:
             msg = 'weave join'
 
         merged = 0
-        processed = 0
         time0 = time.time( )
-        for other_idx, name in enumerate(other._names):
+        for other_idx, name in names_to_join:
             # TODO: If all the parents of the other version are already
             # present then we can avoid some work by just taking the delta
             # and adjusting the offsets.
             new_parents = self._imported_parents(other, other_idx)
             sha1 = other._sha1s[other_idx]
 
-            processed += 1
-
-            if name in self._name_map:
-                idx = self.lookup(name)
-                n1 = set(map(other.idx_to_name, other._parents[other_idx]))
-                n2 = set(map(self.idx_to_name, self._parents[idx]))
-                if sha1 ==  self._sha1s[idx] and n1 == n2:
-                        continue
+            merged += 1
 
             if pb:
-                pb.update(msg, other_idx, len(other._names))
+                pb.update(msg, merged, len(names_to_join))
            
-            merged += 1
             lines = other.get_lines(other_idx)
             self.add(name, new_parents, lines, sha1)
 
         mutter("merged = %d, processed = %d, file_id=%s; deltat=%d"%(
-                merged,processed,self._weave_name, time.time( )-time0))
-
-
-
+                merged, processed, self._weave_name, time.time( )-time0))
  
     def _imported_parents(self, other, other_idx):
         """Return list of parents in self corresponding to indexes in other."""

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2006-02-01 12:46:12 +0000
+++ bzrlib/workingtree.py	2006-02-04 19:38:26 +0000
@@ -745,12 +745,12 @@
                 yield stem
 
     @needs_write_lock
-    def pull(self, source, overwrite=False):
+    def pull(self, source, overwrite=False, stop_revision=None):
         from bzrlib.merge import merge_inner
         source.lock_read()
         try:
             old_revision_history = self.branch.revision_history()
-            count = self.branch.pull(source, overwrite)
+            count = self.branch.pull(source, overwrite,stop_revision)
             new_revision_history = self.branch.revision_history()
             if new_revision_history != old_revision_history:
                 if len(old_revision_history):

